import logging

from odoo import models

from odoo.addons.saas_worker.util import version_gte


_logger = logging.getLogger(__name__)


class PaymentTransaction(models.Model):
    _inherit = 'payment.transaction'

    def _get_specific_processing_values(self, processing_values):
        """ Override of payment_stripe to add the connected account id to the processing values.

        If the provider's publishable key is set, no connect account id is added to use the
        provider's account.

        Note: self.ensure_one() from `_get_processing_values`

        :param dict processing_values: The generic processing values of the transaction
        :return: The dict of provider-specific processing values
        :rtype: dict
        """
        res = super()._get_specific_processing_values(processing_values)

        provider = self._get_provider()
        if provider._get_code() != 'stripe' or self.operation == 'online_token':
            return res

        if not provider.stripe_publishable_key and provider.stripe_account_id:
            res['stripe_account'] = provider.stripe_account_id
        return res

    def _get_common_stripe_session_values(self, pmt_values, customer):
        """ Override of payment_stripe to add the webhook URL metadata to the session values.

        This metadata is used by the proxy to forward session notifications to the database.

        :param dict pmt_values: The payment method types values
        :param dict customer: The Stripe customer to assign to the session
        :return: The common Stripe Session values
        :rtype: dict
        """
        return {
            **super()._get_common_stripe_session_values(pmt_values, customer),
            'metadata[webhook_url]': self._get_provider()._get_stripe_webhook_url(),
        }

    def _stripe_prepare_setup_intent_payload(self, **kwargs):
        """ Override of `payment_stripe` to add the webhook URL metadata to the setup intent values.

        This metadata is used by the proxy to forward notifications to the database.

        :param dict kwargs: Optional keywords arguments from parent method.
        :return: The Stripe-formatted payload for the setup intent request.
        :rtype: dict
        """
        return {
            **super()._stripe_prepare_setup_intent_payload(**kwargs),
            'metadata[webhook_url]': self._get_provider()._get_stripe_webhook_url(),
        }

    def _stripe_prepare_payment_intent_payload(self, **kwargs):
        """ Override of payment_stripe to add the webhook URL metadata to the payment intent values.

        This metadata is used by the proxy to forward payment notifications to the database.

        :param dict kwargs: Optional keywords arguments from parent method.
        :return: The Stripe-formatted payload for the payment intent request
        :rtype: dict
        """
        return {
            **super()._stripe_prepare_payment_intent_payload(**kwargs),
            'metadata[webhook_url]': self._get_provider()._get_stripe_webhook_url(),
        }

    # === HELPER METHODS === #

    def _get_provider(self):
        """ Return the `provider_id` or `acquirer_id` field depending on whether the database is
        running in Odoo 16.0 or lower.

        :return: The provider handling the transaction.
        :rtype: recordset of `payment.acquirer` (16.0+) or of `payment.acquirer` (<16.0)
        """
        if version_gte('16.0'):
            return self.provider_id
        else:
            return self.acquirer_id
